#    ___  _ _                         ____________  _____
#   / _ \| | |                        | ___ \ ___ \/  __ \
#  / /_\ \ | |_ ___  __ _ _ __ _ __   | |_/ / |_/ /| /  \/
#  |  _  | | __/ _ \/ _` | '__| '_ \  |    /|  __/ | |
#  | | | | | ||  __/ (_| | |  | | | | | |\ \| |    | \__/\
#  \_| |_/_|\__\___|\__,_|_|  |_| |_| \_| \_\_|     \____/
#
#

# Durée entre chaque changement de statut (en secondes)
# /!\ Discord impose 15s minimum entre chaque changement
change_time = 60

# Durée entre chaque tentative de connection aprés une erreur
# Mettre ce temps trés bas n'est pas recommandé, surtout si retry_number est sur 0
retry_time = 60

# Nombre de tentatives de connexion échouées à la suite avant de quitter le programme
# Définir à 0 pour désactiver
retry_number = 5

# Noms des différents éléments à afficher
names = ["Altearn", "Gunivers", "Curiosity"]

# IDs de clients des éléments
ids = {"Altearn": "844964834880782357",
       "Gunivers": "845325102551924756",
       "Curiosity": "845325302435414046"}

# Slogans des éléments
states = {"Gunivers": "Creativity begins here !",
          "Altearn": "Apprendre, autrement !",
          "Curiosity": "Apprennez sans cesse !"}

# Images de jour pour chaque élément
large_image = {"Altearn": "altearn_white",
               "Gunivers": "gunivers",
               "Curiosity": "curiosity"}

# Images de nuit pour chaque élément
large_image_night = {"Altearn": "altearn_black",
                     "Gunivers": "gunivers",
                     "Curiosity": "curiosity"}

# Boutons pour chaque éléments
buttons = {"Altearn": [{"label": "En savoir plus", "url": "https://altearn.xyz/"},
                       {"label": "Wiki", "url": "https://wiki.altearn.xyz"}],
           "Gunivers": [{"label": "En savoir plus", "url": "https://gunivers.net/"},
                        {"label": "Discord", "url": "https://discord.gg/E8qq6tN"}],
           "Curiosity": [{"label": "Discord", "url": "https://discord.gg/8Dub7zVvV9"}]}

# Citations aléatoires
quotes = [
    "Luxembourg -- Nemesis, 2021",
    "C'est le top ça -- Leirof, 2021",
    "Je s'appelle groot -- Fleaudesme, 2021",
    "Pain au chocolat ou chocolatine ? -- 301, 2017",
    "Fromage -- Fantomitechno, 2021",
    "On peut être efficace (sérieux c'est moins sur) -- Leirof, 2021",
    "Never gonna give you up -- Rick Asley"
]



import pathlib

from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="altearnrpc",
    version="0.1.4",
    description="Un client Discord RPC pour Altearn",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/Aeris1One/altearn-discord-rpc",
    author="Aeris One",
    author_email="aeris@e.email",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["altearnrpc"],
    include_package_data=True,
    install_requires=["pypresence", "tqdm"],
    entry_points={
        "console_scripts": [
            "altearnrpc=altearnrpc.__main__:main",
        ]
    },
)



import argparse
import os
import time
from datetime import datetime
from random import choice
from sys import platform

from pypresence import Presence
from tqdm import tqdm

from altearnrpc import config

parser = argparse.ArgumentParser()
parser.add_argument("--quiet", help="désactiver le gui, utile pour faire tourner le système en arrière-plan",
                    action="store_true")
parser.add_argument("--endless", help="désactiver la limitation d'essais infructueux de connection",
                    action="store_true")
args = parser.parse_args()
if args.quiet:
    print("verbosity turned on")


def clear_console():
    if platform == "linux" or platform == "darwin":
        os.system('clear')
    elif platform == "win32":
        os.system('cls')


def altearn_rpc():
    clear_console()
    if not args.quiet:
        print("    ___  _ _                         ____________  _____")
        print("   / _ \| | |                        | ___ \ ___ \/  __ \ ")
        print("  / /_\ \ | |_ ___  __ _ _ __ _ __   | |_/ / |_/ /| /  \/")
        print("  |  _  | | __/ _ \/ _` | '__| '_ \  |    /|  __/ | |")
        print("  | | | | | ||  __/ (_| | |  | | | | | |\ \| |    | \__/\ ")
        print("  \_| |_/_|\__\___|\__,_|_|  |_| |_| \_| \_\_|     \____/")
        print("")


def main():
    error_nbr = 0
    used_names = []
    while True:
        try:
            # Choisir un élément à afficher qui ne soit pas dans la liste des statuts déjà affichés
            name = choice(config.names)
            while name in used_names:
                name = choice(config.names)
            used_names.append(name)

            # Si les deux listes sont les mêmes réinitialiser used_names, tous les statuts sont passés
            list_a = set(config.names)
            list_b = set(used_names)
            if list_a == list_b:
                used_names = []

            # Récupérer ses informations
            client_id = config.ids[name]
            st = config.states[name]
            bu = config.buttons[name]
            lt = choice(config.quotes)

            # Se connecter au serveur RPC Discord
            RPC = Presence(client_id)
            RPC.connect()

            # Choisir entre le logo jour et le logo nuit
            if datetime.now().hour >= 19 or datetime.now().hour <= 7:
                li = config.large_image_night[name]
            else:
                li = config.large_image[name]

            # Envoyer les informations au RPC Discord
            RPC.update(state=st,
                       buttons=bu,
                       large_image=li,
                       large_text=lt)

            # Afficher une interface basique
            if not args.quiet:
                time.sleep(config.change_time)
            else:
                altearn_rpc()
                print("En cours d'affichage: " + str(name))
                print("Citation: " + str(lt))
                print("")
                print("Prochain changement :")

                # Attendre 60 secondes avec un barre de chargement
                for _ in tqdm(range(int(config.change_time // 0.2)), bar_format="|{bar}|{remaining}{postfix}"):
                    time.sleep(0.2)

            # Fermer la connection actuelle
            RPC.close()

            error_nbr = 0

        except KeyboardInterrupt:
            print("Fermeture de la connection RPC...")
            RPC.close()
            exit()

        except ConnectionRefusedError:
            if not args.endless:
                if config.retry_number >= 2:
                    if error_nbr == config.retry_number:
                        altearn_rpc()
                        print(
                            "Arrêt du programme aprés" + config.retry_number + "tentatives infructueuses de connection à Discord.")
                        exit()
                elif config.retry_number == 1:
                    if error_nbr == config.retry_number:
                        altearn_rpc()
                        print("Arrêt du programme aprés une tentative infructueuse de connection à Discord.")
                        exit()
            else:
                if not args.quiet:
                    time.sleep(config.retry_time)
                else:
                    error_nbr += 1
                    altearn_rpc()
                    print("Impossible de se connecter à Discord... (Tentative n°" + str(error_nbr) + ")")
                    print("Avez-vous le client allumé ?")
                    print(" ")
                    print("Nouvelle tentative dans :")
                    for _ in tqdm(range(int(config.retry_time // 0.2)), bar_format="|{bar}|{remaining}{postfix}"):
                        time.sleep(0.2)


if __name__ == "__main__":
    main()
